/*
 * Copyright 2009 © David Casperson
 * May be used freely, provided that this notice is preserved.
 */

#include <cstdlib>              // for abort()
#include "stopwatch.h"
#include "system_time.h"
#include <iostream>


// anonymous helper functions


StopWatch::StopWatch()
    : running(false)
{
    reset() ;
}

void
StopWatch::handleButton(
    system_time const& atTime,
        bool reset,
        bool newRunning)
{
    if (reset)
        this->total = 0.0;
    else if (running)
        this->total += (atTime-epoch).toSeconds() ;
    epoch = atTime ;
    this->running = newRunning ;
    return ;
}

StopWatch&
StopWatch::start()
{
    handleButton(system_time(),false,true) ;
    return *this ;
}

StopWatch&
StopWatch::stop()
{
    handleButton(system_time(),false,false) ;
    return *this ;
}

StopWatch&
StopWatch::reset()
{
    handleButton(system_time(),true,is_running()) ;
    return *this ;
}

double
StopWatch::elapsed() const
{
    return 
        (is_running() ? (system_time()-epoch).toSeconds() : 0.0)
        + total ;
}

bool
StopWatch::is_running() const
{
    return this->running ;
}



std::ostream& operator<<(std::ostream& out, const StopWatch& s)
{
    return out << s.elapsed()*1000.0 << "ms " ;
}
