//                                         This file is really -*-C++/l-*-
//          Project:  Code/C++/Stopwatch/
//             File:  system_time.c++
//       Created by:  David Casperson
//          Created:  Fri Nov  2 13:19:56 2007
//
//      Description:  system time implementation for a Macintosh

/*
 * Copyright 2009 © David Casperson
 * May be used freely, provided that this notice is preserved.
 */

#include "system_time.h"

#include <sys/resource.h>
#include <cstdlib>


namespace 
{
    long int const one_billion = 1000 * 1000 * 1000L ;

    
    void normalize (timespec & t)
    {
        if (t.tv_nsec < 0)
        do
            {
            t.tv_nsec += one_billion ;
            t.tv_sec -= 1 ;
            } while (t.tv_nsec < 0) ;
        else
            {
            t.tv_sec += (t.tv_nsec / one_billion);
            t.tv_nsec %= one_billion ;
            }
    }

    void add3(timespec const& summand1,
              timespec const& summand2,
              timespec      & answer)
    {
        int const one_M = 1000000 ;
        int const one_k = 1000 ;
        answer.tv_sec  = summand1.tv_sec  + summand2.tv_sec ;
        int temp_ms = summand1.tv_nsec / one_M
                    + summand2.tv_nsec / one_M ;
        answer.tv_nsec =
            (summand1.tv_nsec % one_M )
            + ( summand2.tv_nsec % one_M ) ;
        answer.tv_nsec += one_M * ( temp_ms % one_k ) ;
        answer.tv_sec  += ( temp_ms / one_k ) ;
        normalize(answer) ;
    }

    void subtract3(timespec const& summand1,
                   timespec const& summand2,
                   timespec      & answer)
    {
        answer.tv_sec  = - summand2.tv_sec ;
        answer.tv_nsec = - summand2.tv_nsec ;
        add3(answer, summand1, answer) ;
        normalize(answer) ;
    }

    double ts_TO_SECONDS(timespec const& v)
    {
        return v.tv_nsec / 1.0E9 + v.tv_sec / 1.0 ;
    }

    void zero(timespec& v)
    {
        v.tv_sec  = 0 ;
        v.tv_nsec = 0 ;
    }
}

system_time::DontInitializeClass system_time::DontInitialize ;


system_time::system_time()
{
    struct rusage my_usage ;
    if (-1==getrusage (RUSAGE_SELF,&my_usage))
        {
        abort() ;
        }
    TIMEVAL_TO_TIMESPEC(&my_usage.ru_utime,&this->time1) ;
    return ;
}

system_time
system_time::operator-(system_time const& rhs) const
{
    system_time answer(DontInitialize) ;
    subtract3(this->time1,rhs.time1,answer.time1) ;
    return answer ;
}

double system_time::toSeconds() const
{
    return ts_TO_SECONDS(this->time1) ;
}

